# This file is here for PAUSE and will be replaced at build time
package Object::Iterate;
$VERSION = '1.153';

=head1 NAME

Object::Iterate - iterators for objects that know the next element

=cut

1;
